# ../addons/eventscripts/gungame51/scripts/custom/gg_assist_config.py

# ============================================================================
# >> IMPORTS
# ============================================================================
# EventScripts Imports
import es
import cfglib

# GunGame Imports
from gungame51.core.cfg import generate_header

# ============================================================================
# >> GLOBAL VARIABLES
# ============================================================================
config = cfglib.AddonCFG('%s/cfg/' %es.ServerVar('eventscripts_gamedir') +
    'gungame51/custom_addon_configs/gg_assist.cfg')
        
# ============================================================================
# >> LOAD & UNLOAD
# ============================================================================
def load():
    generate_header(config)
    
    # GG Assist
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST')
    config.text('='*76)
    config.text('Description:')
    config.text('   Awards assist points to the players that damaged the ' +
                'player, but did not')
    config.text('   kill the player. 100 assist points can be redeemed for a' +
        ' level.')
    config.text('Options:')
    config.text('   0 = (Disabled) Do not load gg_assist.')
    config.text('   1 = (Enabled) Load gg_assist.')
    config.text('Default Value: 0')
    config.cvar('gg_assist', 0, 'Enables/Disables gg_assist.').addFlag('notify')

    # GG Assist Percent
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST PERCENT')
    config.text('='*76)
    config.text('Description:')
    config.text('   The percentage of points to be awarded for damage done' +
                ' to players.')
    config.text('Example:')
    config.text('   If a player does 93 damage with gg_assist_percent set at' +
        ' 10%, 9.3 assist')
    config.text('   points would be awarded to that player.')
    config.text('Options:')
    config.text('   # = any integer value greater than 0')
    config.text('Default Value: 10')
    config.cvar('gg_assist_percent', 10, 'Percentage of damage to award as ' +
        'points.')

    # GG Assist Award Alive Only
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST AWARD ALIVE ONLY')
    config.text('='*76)
    config.text('Description:')
    config.text('   Awards assist points to only living players.')
    config.text('Options:')
    config.text('   0 = Award assist points to living AND dead players.')
    config.text('   1 = Award assist points ONLY to living players.')
    config.text('Default Value: 0')
    config.cvar('gg_assist_award_alive_only', 0, 'Awards assist points only ' +
        'to living players.')

    # GG Assist Skip Knife
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST SKIP KNIFE')
    config.text('='*76)
    config.text('Description:')
    config.text('   Allows player to use assist points to skip knife level.')
    config.text('Options:')
    config.text('   0 = Do not allow players to skip knife level.')
    config.text('   1 = Allow players to skip knife level.')
    config.text('Default Value: 0')
    config.cvar('gg_assist_skip_knife', 0, 'Allows player to use assist ' +
        'points to skip knife level.')

    # GG Assist Skip Nade
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST SKIP NADE')
    config.text('='*76)
    config.text('Description:')
    config.text('   Allows player to use assist points to skip nade level.')
    config.text('Options:')
    config.text('   0 = Do not allow players to skip nade level.')
    config.text('   1 = Allow players to skip nade level.')
    config.text('Default Value: 0')
    config.cvar('gg_assist_skip_nade', 0, 'Allows player to use assist ' +
        'points to skip nade level.')

    # GG Assist Redeem Win
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST REDEEM WIN')
    config.text('='*76)
    config.text('Description:')
    config.text('   Allows player to use assist points to win.')
    config.text('Options:')
    config.text('   0 = Do not allow players to win by redeeming assist ' +
        'points.')
    config.text('   1 = Allow players to win by redeeming assist points.')
    config.text('Default Value: 0')
    config.cvar('gg_assist_redeem_win', 0, 'Allows player to use assist ' +
        'points to win.')

    # GG Assist Advertisement
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST ADVERTISEMENT')
    config.text('='*76)
    config.text('Description:')
    config.text('   When a player earns assist points, the player will ' +
        'be told privately how')
    config.text('   many assist points were earned, and their assist point ' +
        'balance.')
    config.text('Options:')
    config.text('   0 = Do not advertise assist points when earned.')
    config.text('   1 = Advertise assist points when earned.')
    config.text('Default Value: 1')
    config.cvar('gg_assist_advertise', 1, 'Advertises assist points to the ' +
        'players when earned.')

    # GG Assist Sound
    config.text('')
    config.text('='*76)
    config.text('>> GG ASSIST SOUND')
    config.text('='*76)
    config.text('Description:')
    config.text('   Sound that plays when 100 points are accumulated.')
    config.text('Options:')
    config.text('   * Path to sound located in the sound directory')
    config.text('   * 0 for no sound')
    config.text('Default Value: "buttons/weapon_confirm.wav"')
    config.cvar('gg_assist_sound', 'buttons/weapon_confirm.wav', 'Sound ' +
        'that plays when 100 points are accumulated.')

    config.write()
    es.dbgmsg(0, '\tgg_assist.cfg')

def unload():
    global config

    # Remove the "notify" flags as set by addFlag('notify')
    for cvar in config.getCvars().keys():
        es.flags('remove', 'notify', cvar)
    
    # Delete the cfglib.AddonCFG instance
    del config